<?php

namespace Drupal\Tests\zoomapi\Functional;

use Drupal\Core\Url;
use Drupal\key\Entity\Key;
use Drupal\Tests\BrowserTestBase;

/**
 * Simple test to ensure that the config page loads.
 *
 * @group zoomapi
 */
class ConfigLoadTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = ['zoomapi', 'key'];

  /**
   * A user with permission to administer site configuration.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * User with content access.
   *
   * @var \Drupal\user\Entity\User|false
   */
  protected $normalUser;

  /**
   * A key entity to use for testing.
   *
   * @var \Drupal\key\KeyInterface
   */
  protected $testKey;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->adminUser = $this->drupalCreateUser(['administer zoom api']);
    $this->drupalLogin($this->adminUser);
    // Create Key module keys.
    // Api Key.
    $this->createTestKey(
      'test_zoomapi_key',
      'authentication',
      'config'
    );

    // API Secret.
    $this->createTestKey(
      'test_zoomapi_secret',
      'authentication',
      'config'
    );
  }

  /**
   * Tests that the config page loads and functions.
   */
  public function testConfigPage() {
    $this->drupalGet(Url::fromRoute('zoomapi.settings'));
    $this->assertSession()->pageTextContains('Zoom API Settings');
    $this->assertSession()->statusCodeEquals(200);

    $edit = [
      'base_uri' => 'https://api.zoom.us/v2222222',
      'api_key' => 'test_zoomapi_key',
      'api_secret' => 'test_zoomapi_secret',
    ];
    $this->submitForm($edit, 'Save configuration');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertEquals($edit['base_uri'], $this->config('zoomapi.settings')->get('base_uri'), 'Config form is properly saving values.');
    ;
  }

  /**
   * Checks if a non-administrative user cannot access to the config page.
   */
  public function testAccessDenied() {
    // Create user for testing.
    $this->normalUser = $this->drupalCreateUser(['access content']);

    // Login for the former basic user.
    $this->drupalLogin($this->normalUser);
    $this->drupalGet(Url::fromRoute('zoomapi.settings'));

    // Check the response returned by Drupal.
    $this->assertSession()->statusCodeEquals(403);
  }

  /**
   * Make a key for testing operations that require a key.
   */
  protected function createTestKey($id, $type = NULL, $provider = NULL) {
    $keyArgs = [
      'id' => $id,
      'label' => "Zoom API Webhook Test Key ($id)",
    ];
    if ($type != NULL) {
      $keyArgs['key_type'] = $type;
    }
    if ($provider != NULL) {
      $keyArgs['key_provider'] = $provider;
    }
    $this->testKey = Key::create($keyArgs);
    $this->testKey->setKeyValue('taco');
    $this->testKey->save();
    return $this->testKey;
  }

}
